@@ -3,7 +3,7 @@ use crate::nixpacks::{
     app,
     environment::Environment,
     images::DEFAULT_BASE_IMAGE,
-    nix,
+    nix::{create_nix_expressions_for_phases, nix_file_names_for_phases},
     plan::{
         phase::{Phase, StartPhase},
         BuildPlan,
@@ -101,6 +101,33 @@ impl DockerfileGenerator for BuildPlan {
     ) -> Result<String> {
         let plan = self;
 
+        let nix_file_names = nix_file_names_for_phases(&plan.phases.clone().unwrap_or_default());
+
+        let mut nix_install_cmds: Vec<String> = Vec::new();
+        for name in nix_file_names {
+            let nix_file = output.get_relative_path(name);
+
+            let nix_file_path = nix_file
+                .to_slash()
+                .context("Failed to convert nix file path to slash path.")?;
+
+            nix_install_cmds.push(format!(
+                "COPY {nix_file_path} {nix_file_path}\nRUN nix-env -if {nix_file_path} && nix-collect-garbage -d",
+                nix_file_path = nix_file_path
+            ));
+        }
+        let nix_install_cmds = nix_install_cmds.join("\n");
+
+        let apt_pkgs = self.all_apt_packages();
+        let apt_pkgs_str = if apt_pkgs.is_empty() {
+            "".to_string()
+        } else {
+            format!(
+                "RUN apt-get update && apt-get install -y --no-install-recommends {}",
+                apt_pkgs.join(" ")
+            )
+        };
+
         let variables = plan.variables.clone().unwrap_or_default();
         let args_string = if variables.is_empty() {
             "".to_string()
@@ -136,7 +163,6 @@ impl DockerfileGenerator for BuildPlan {
         let phases = plan.get_sorted_phases()?;
 
         let dockerfile_phases = phases
-            .clone()
             .into_iter()
             .map(|phase| {
                 let phase_dockerfile =
@@ -147,30 +173,10 @@ impl DockerfileGenerator for BuildPlan {
                             phase.get_name()
                         ))?;
 
-                match phase.get_name().as_str() {
-                    // We want to load the variables immediately after the setup phase
-                    "setup" => Ok(format!(
-                        "{}\n# load variables\n{}\n",
-                        phase_dockerfile, args_string
-                    )),
-                    _ => Ok(phase_dockerfile),
-                }
+                Ok(phase_dockerfile)
             })
-            .collect::<Result<Vec<_>>>();
-        let dockerfile_phases_str = dockerfile_phases?.join("\n");
-
-        // Handle the case where there is only a setup phase and all the app files have not been copied into
-        // the image yet
-        let setup_copy_command = match (phases.first(), phases.len()) {
-            (Some(first_phase), 1) => {
-                if first_phase.get_name() == "setup" {
-                    utils::get_copy_command(&[".".to_string()], APP_DIR)
-                } else {
-                    "".to_string()
-                }
-            }
-            _ => "".to_string(),
-        };
+            .collect::<Result<Vec<_>>>()?;
+        let dockerfile_phases_str = dockerfile_phases.join("\n");
 
         let start_phase_str = plan
             .start_phase
@@ -189,18 +195,22 @@ impl DockerfileGenerator for BuildPlan {
             ENTRYPOINT [\"/bin/bash\", \"-l\", \"-c\"]
             WORKDIR {APP_DIR}
 
+            {nix_install_cmds}
+            {apt_pkgs_str}
             {assets_copy_cmd}
+            {args_string}
 
             {dockerfile_phases_str}
 
-            {setup_copy_command}
             {start_phase_str}
         ", 
         base_image=base_image,
         APP_DIR=APP_DIR,
+        nix_install_cmds=nix_install_cmds,
+        apt_pkgs_str=apt_pkgs_str,
         assets_copy_cmd=assets_copy_cmd,
+        args_string=args_string,
         dockerfile_phases_str=dockerfile_phases_str,
-        setup_copy_command=setup_copy_command,
         start_phase_str=start_phase_str};
 
         Ok(dockerfile)
@@ -214,6 +224,17 @@ impl DockerfileGenerator for BuildPlan {
     ) -> Result<()> {
         self.write_assets(self, output).context("Writing assets")?;
 
+        let nix_expressions =
+            create_nix_expressions_for_phases(&self.phases.clone().unwrap_or_default());
+
+        for (name, nix_expression) in nix_expressions {
+            let nix_path = output.get_absolute_path(name);
+            let mut nix_file = File::create(nix_path).context("Creating Nix environment file")?;
+            nix_file
+                .write_all(nix_expression.as_bytes())
+                .context("Unable to write Nix expression")?;
+        }
+
         for phase in self.get_sorted_phases()? {
             phase
                 .write_supporting_files(options, env, output)
@@ -246,6 +267,15 @@ impl BuildPlan {
 
         Ok(())
     }
+
+    fn all_apt_packages(&self) -> Vec<String> {
+        self.phases
+            .clone()
+            .unwrap_or_default()
+            .values()
+            .flat_map(|phase| phase.apt_pkgs.clone().unwrap_or_default())
+            .collect()
+    }
 }
 
 impl DockerfileGenerator for StartPhase {
@@ -301,7 +331,7 @@ impl DockerfileGenerator for Phase {
         &self,
         options: &DockerBuilderOptions,
         env: &Environment,
-        output: &OutputDir,
+        _output: &OutputDir,
     ) -> Result<String> {
         let phase = self;
 
@@ -325,37 +355,9 @@ impl DockerfileGenerator for Phase {
             ("".to_string(), "".to_string())
         };
 
-        // Install nix packages and libraries
-        let install_nix_pkgs_str = if self.uses_nix() {
-            let nix_file = output.get_relative_path(format!("{}.nix", phase.get_name()));
-
-            let nix_file_path = nix_file
-                .to_slash()
-                .context("Failed to convert nix file path to slash path.")?;
-            format!(
-                "COPY {nix_file_path} {nix_file_path}\nRUN nix-env -if {nix_file_path} && nix-collect-garbage -d",
-                nix_file_path = nix_file_path
-            )
-        } else {
-            "".to_string()
-        };
-
-        // Install apt packages
-        let apt_pkgs = phase.apt_pkgs.clone().unwrap_or_default();
-        let apt_pkgs_str = if apt_pkgs.is_empty() {
-            "".to_string()
-        } else {
-            format!(
-                "RUN apt-get update && apt-get install -y {}",
-                apt_pkgs.join(" ")
-            )
-        };
-
         // Copy over app files
         let phase_files = match (phase.get_name().as_str(), &phase.only_include_files) {
             (_, Some(files)) => files.clone(),
-            // Special case for the setup phase, which has no files
-            ("setup", None) => vec![],
             _ => vec![".".to_string()],
         };
         let phase_copy_cmd = utils::get_copy_command(&phase_files, APP_DIR);
@@ -370,18 +372,11 @@ impl DockerfileGenerator for Phase {
             .collect::<Vec<_>>()
             .join("\n");
 
-        let dockerfile_stmts = vec![
-            build_path,
-            run_path,
-            install_nix_pkgs_str,
-            apt_pkgs_str,
-            phase_copy_cmd,
-            cmds_str,
-        ]
-        .into_iter()
-        .filter(|stmt| !stmt.is_empty())
-        .collect::<Vec<_>>()
-        .join("\n");
+        let dockerfile_stmts = vec![build_path, run_path, phase_copy_cmd, cmds_str]
+            .into_iter()
+            .filter(|stmt| !stmt.is_empty())
+            .collect::<Vec<_>>()
+            .join("\n");
 
         let dockerfile = formatdoc! {"
             # {name} phase
@@ -398,20 +393,8 @@ impl DockerfileGenerator for Phase {
         &self,
         _options: &DockerBuilderOptions,
         _env: &Environment,
-        output: &OutputDir,
+        _output: &OutputDir,
     ) -> Result<()> {
-        if self.uses_nix() {
-            // Write the Nix expressions to the output directory
-            let nix_file_name = format!("{}.nix", self.get_name());
-            let nix_path = output.get_absolute_path(nix_file_name);
-            let nix_expression = nix::create_nix_expression(self);
-
-            let mut nix_file = File::create(nix_path).context("Creating Nix environment file")?;
-            nix_file
-                .write_all(nix_expression.as_bytes())
-                .context("Unable to write Nix expression")?;
-        }
-
         Ok(())
     }
 }
@@ -435,8 +418,6 @@ mod tests {
             .unwrap();
 
         assert!(dockerfile.contains("echo test"));
-        assert!(dockerfile.contains("apt-get update"));
-        assert!(dockerfile.contains("wget"));
     }
 
     #[test]
@@ -445,6 +426,7 @@ mod tests {
 
         let mut test1 = Phase::new("test1");
         test1.add_cmd("echo test1");
+        test1.add_apt_pkgs(vec!["wget".to_owned()]);
         plan.add_phase(test1);
 
         let mut test2 = Phase::new("test2");
@@ -461,5 +443,7 @@ mod tests {
 
         assert!(dockerfile.contains("echo test1"));
         assert!(dockerfile.contains("echo test2"));
+        assert!(dockerfile.contains("apt-get update"));
+        assert!(dockerfile.contains("wget"));
     }
 }
